import { createInstance, INDEXEDDB } from 'localforage'
import { reactive, toRaw, watch, type WatchStopHandle } from 'vue'

const db = createInstance({
  driver: INDEXEDDB,
  name: 'fatfox',
  version: 1.0,
  storeName: 'fat_fox_key_value_pairs'
})

export function useForageStore<T extends { [key: string]: any; loading: boolean }>(
  name: string,
  defaultData: T,
  partialWrite: (res: T) => T = (res) => res
) {
  const state = reactive(defaultData)
  const writeWatch = () =>
    watch(
      state,
      (d) => {
        db.setItem(name, partialWrite(toRaw(d) as T))
      },
      { deep: true }
    )
  let stopWatch: WatchStopHandle = () => {}
  const refresh = () => {
    stopWatch()
    state.loading = true
    db.getItem<{ data: T }>(name).then((res) => {
      if (res?.data) {
        Object.assign(state, res.data)
      }
      state.loading = false
      stopWatch = writeWatch()
    })
  }
  refresh()
  document.addEventListener('visibilitychange', () => {
    if (document.visibilityState === 'visible') {
      refresh()
    }
  })
}

export default db
